/*
 * Copyright (c) 2015, Majenko Technologies
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * * Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.
 *
 * * Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * * Neither the name of Majenko Technologies nor the names of its
 *   contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <genieArduino.h>
#include <SoftwareSerial.h>

#define RESETLINE 4  // Change this if you are not using an Arduino Adaptor Shield Version 2 (see code below)

Genie genie;

// Use software serial on pins 10 and 11, which means set the RX and TX jumpers on
// the Arduino breakout board to use D10 and D11.
SoftwareSerial serial(10, 11);

int counter;
char charArray1[] = {'l','o','o','p',':',' '};
char charArray2[] = {' ','t','i','m','e','(','s',')','\0'};
char bufIntToChar[12];  //create a character array for storing the integer-to-string value to be printed
char finalArray[25]; // this will contain the final string to be printed


void setup()
{
  Serial.begin(9600);  // Console in Arduino IDE on Serial0
  serial.begin(9600);  
  genie.Begin(serial); // display connected to a software serial port

  genie.AttachEventHandler(myGenieEventHandler); // Attach the user function Event Handler for processing events
  // Reset the Display (change D4 to D2 if you have original 4D Arduino Adaptor)
  // THIS IS IMPORTANT AND CAN PREVENT OUT OF SYNC ISSUES, SLOW SPEED RESPONSE ETC
  // If NOT using a 4D Arduino Adaptor, digitalWrites must be reversed as Display Reset is Active Low, and
  // the 4D Arduino Adaptors invert this signal so must be Active High.
  pinMode(RESETLINE, OUTPUT);  // Set D4 on Arduino to Output (4D Arduino Adaptor V2 - Display Reset)
  digitalWrite(RESETLINE, 1);  // Reset the Display via D4
  delay(100);
  digitalWrite(RESETLINE, 0);  // unReset the Display via D4  
  delay (5000); //let the display start up after the reset (This is important)  

  Serial.println("Hello World!");

}

void loop()
{
  static long waitPeriod = millis();
  int i = 0;
  char x;
  
  genie.DoEvents(); // This calls the library each loop to process the queued responses from the display   
  if (millis() >= waitPeriod){
    
    //integer to character array, decimal. Visit the Arduino website for more information.
    itoa(counter, bufIntToChar, 10);        
    
    //determine the number of digits 
    do{
      x = bufIntToChar[i];
      i++;      
    }while(x != '\0');
    i--;
    
    //go to the Arduino website for more information on the function "memcpy()"
    memcpy(finalArray, charArray1, sizeof(charArray1));  //append charArray1 to the final array    
    memcpy(finalArray + sizeof(charArray1) , bufIntToChar, i); //append the value of the counter to the final array 
    memcpy(finalArray + sizeof(charArray1) + i , charArray2, sizeof(charArray2)); //append charArray2 to the final array  
    
    Serial.println(finalArray);      //print the final array on the serial monitor
    genie.WriteStr(0, bufIntToChar); //print the value of the counter on the display
    genie.WriteStr(1, finalArray);   //print the final array on the display

    counter++;
    waitPeriod = millis() + 1000;                    // rerun this code in another 1000ms time.  
  }       

}

/////////////////////////////////////////////////////////////////////
//
// This is the user's event handler. It is called by genieDoEvents()
// when the following conditions are true
//
//		The link is in an IDLE state, and
//		There is an event to handle
//
// The event can be either a REPORT_EVENT frame sent asynchronously
// from the display or a REPORT_OBJ frame sent by the display in
// response to a READ_OBJ request.
//

// LONG HAND VERSION, MAYBE MORE VISIBLE AND MORE LIKE VERSION 1 OF THE LIBRARY
void myGenieEventHandler(void)
{
  genieFrame Event;
  genie.DequeueEvent(&Event);
  // do nothing else
}



